import { __decorate, __param } from 'tslib';
import { ViewContainerRef, TemplateRef, Inject, Renderer2, Input, Directive, NgModule } from '@angular/core';
import { DOCUMENT } from '@angular/common';

const DEFAULT_OFFSET = 0;
const DEFAULT_STICKY_CLASS = 'sticky';
let StickyDirective = class StickyDirective {
    constructor(container, template, document, renderer) {
        this.container = container;
        this.template = template;
        this.document = document;
        this.renderer = renderer;
    }
    get stickyStyle() {
        return this.sticky.rootNodes[0].style;
    }
    get fixedStyle() {
        return this.fixed.rootNodes[0].style;
    }
    get fixedClientRect() {
        return this.fixed.rootNodes[0].getBoundingClientRect();
    }
    get stickyClientRect() {
        return this.sticky.rootNodes[0].getBoundingClientRect();
    }
    get verticalOffset() {
        var _a;
        return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.verticalOffset) || DEFAULT_OFFSET;
    }
    get horizontalOffset() {
        var _a;
        return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.horizontalOffset) || DEFAULT_OFFSET;
    }
    get stickyClass() {
        var _a;
        return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.stickyClass) || DEFAULT_STICKY_CLASS;
    }
    ngOnInit() {
        this.recreateElements();
        this.initializeStickyStyles();
        this.onScrollEventListener = this.renderer.listen(this.document, 'scroll', this.onScroll.bind(this));
    }
    ngOnDestroy() {
        this.onScrollEventListener();
    }
    ngDoCheck() {
        !this.isSizeAndPositionValid() && this.recalculateStickyStyles();
    }
    isSizeAndPositionValid() {
        return this.fixedClientRect.width === this.stickyClientRect.width &&
            this.fixedClientRect.height === this.stickyClientRect.height &&
            (this.fixedClientRect.left === this.stickyClientRect.left || this.stickyClientRect.left === this.horizontalOffset) &&
            (this.fixedClientRect.top === this.stickyClientRect.top || this.stickyClientRect.top === this.verticalOffset);
    }
    recreateElements() {
        this.fixed = this.container.createEmbeddedView(this.template);
        this.sticky = this.container.createEmbeddedView(this.template);
        this.sticky.rootNodes[0].classList.add(this.stickyClass);
    }
    initializeStickyStyles() {
        this.fixedStyle.visibility = 'visible';
        this.stickyStyle.position = 'fixed';
        this.stickyStyle.display = 'none';
        setTimeout(() => this.recalculateStickyStyles());
    }
    shouldShowSticky() {
        return this.fixedClientRect.top < this.verticalOffset || this.fixedClientRect.left < this.horizontalOffset;
    }
    shouldHideSticky() {
        return this.fixedClientRect.top >= this.verticalOffset || this.fixedClientRect.left >= this.horizontalOffset;
    }
    showSticky() {
        this.stickyStyle.display = this.fixedStyle.display;
        this.fixedStyle.visibility = 'hidden';
    }
    hideSticky() {
        this.stickyStyle.display = 'none';
        this.fixedStyle.visibility = 'visible';
    }
    onScroll() {
        if (this.shouldShowSticky()) {
            this.showSticky();
        }
        else if (this.shouldHideSticky()) {
            this.hideSticky();
        }
    }
    recalculateStickyStyles() {
        this.stickyStyle.width = `${this.fixedClientRect.width}px`;
        this.stickyStyle.height = `${this.fixedClientRect.height}px`;
        this.stickyStyle.left = `${Math.max(this.horizontalOffset, this.fixedClientRect.left)}px`;
        this.stickyStyle.top = `${Math.max(this.verticalOffset, this.fixedClientRect.top)}px`;
    }
};
StickyDirective.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: TemplateRef },
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: Renderer2 }
];
__decorate([
    Input('scSticky')
], StickyDirective.prototype, "config", void 0);
StickyDirective = __decorate([
    Directive({
        selector: '[scSticky]'
    }),
    __param(2, Inject(DOCUMENT))
], StickyDirective);

const TH_SELECTOR = 'tr th';
let StickyTheadDirective = class StickyTheadDirective extends StickyDirective {
    constructor(container, template, document, renderer) {
        super(container, template, document, renderer);
        this.container = container;
        this.template = template;
        this.document = document;
        this.renderer = renderer;
    }
    initializeStickyStyles() {
        this.stickyStyle.display = 'block';
        this.stickyStyle.overflow = 'hidden';
        super.initializeStickyStyles();
    }
    recalculateStickyStyles() {
        super.recalculateStickyStyles();
        const fixedThs = this.fixed.rootNodes[0].querySelectorAll(TH_SELECTOR);
        const stickyThs = this.sticky.rootNodes[0].querySelectorAll(TH_SELECTOR);
        for (let i = 0; i < fixedThs.length; i++) {
            stickyThs[i].style.width = `${fixedThs[i].getBoundingClientRect().width}px`;
            stickyThs[i].style.height = `${fixedThs[i].getBoundingClientRect().height}px`;
            stickyThs[i].style.display = 'inline-block';
        }
    }
};
StickyTheadDirective.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: TemplateRef },
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: Renderer2 }
];
__decorate([
    Input('scStickyThead')
], StickyTheadDirective.prototype, "stickyOffset", void 0);
StickyTheadDirective = __decorate([
    Directive({
        selector: '[scStickyThead]'
    }),
    __param(2, Inject(DOCUMENT))
], StickyTheadDirective);

let NgxStickyCopyModule = class NgxStickyCopyModule {
};
NgxStickyCopyModule = __decorate([
    NgModule({
        declarations: [StickyDirective, StickyTheadDirective],
        imports: [],
        exports: [StickyDirective, StickyTheadDirective]
    })
], NgxStickyCopyModule);

/*
 * Public API Surface of ngx-sticky-copy
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxStickyCopyModule, StickyDirective, StickyTheadDirective };
//# sourceMappingURL=ngx-sticky-copy.js.map
