import { __decorate, __param, __extends } from 'tslib';
import { ViewContainerRef, TemplateRef, Inject, Renderer2, Input, Directive, NgModule } from '@angular/core';
import { DOCUMENT } from '@angular/common';

var DEFAULT_OFFSET = 0;
var DEFAULT_STICKY_CLASS = 'sticky';
var StickyDirective = /** @class */ (function () {
    function StickyDirective(container, template, document, renderer) {
        this.container = container;
        this.template = template;
        this.document = document;
        this.renderer = renderer;
    }
    Object.defineProperty(StickyDirective.prototype, "stickyStyle", {
        get: function () {
            return this.sticky.rootNodes[0].style;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StickyDirective.prototype, "fixedStyle", {
        get: function () {
            return this.fixed.rootNodes[0].style;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StickyDirective.prototype, "fixedClientRect", {
        get: function () {
            return this.fixed.rootNodes[0].getBoundingClientRect();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StickyDirective.prototype, "stickyClientRect", {
        get: function () {
            return this.sticky.rootNodes[0].getBoundingClientRect();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StickyDirective.prototype, "verticalOffset", {
        get: function () {
            var _a;
            return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.verticalOffset) || DEFAULT_OFFSET;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StickyDirective.prototype, "horizontalOffset", {
        get: function () {
            var _a;
            return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.horizontalOffset) || DEFAULT_OFFSET;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StickyDirective.prototype, "stickyClass", {
        get: function () {
            var _a;
            return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.stickyClass) || DEFAULT_STICKY_CLASS;
        },
        enumerable: true,
        configurable: true
    });
    StickyDirective.prototype.ngOnInit = function () {
        this.recreateElements();
        this.initializeStickyStyles();
        this.onScrollEventListener = this.renderer.listen(this.document, 'scroll', this.onScroll.bind(this));
    };
    StickyDirective.prototype.ngOnDestroy = function () {
        this.onScrollEventListener();
    };
    StickyDirective.prototype.ngDoCheck = function () {
        !this.isSizeAndPositionValid() && this.recalculateStickyStyles();
    };
    StickyDirective.prototype.isSizeAndPositionValid = function () {
        return this.fixedClientRect.width === this.stickyClientRect.width &&
            this.fixedClientRect.height === this.stickyClientRect.height &&
            (this.fixedClientRect.left === this.stickyClientRect.left || this.stickyClientRect.left === this.horizontalOffset) &&
            (this.fixedClientRect.top === this.stickyClientRect.top || this.stickyClientRect.top === this.verticalOffset);
    };
    StickyDirective.prototype.recreateElements = function () {
        this.fixed = this.container.createEmbeddedView(this.template);
        this.sticky = this.container.createEmbeddedView(this.template);
        this.sticky.rootNodes[0].classList.add(this.stickyClass);
    };
    StickyDirective.prototype.initializeStickyStyles = function () {
        var _this = this;
        this.fixedStyle.visibility = 'visible';
        this.stickyStyle.position = 'fixed';
        this.stickyStyle.display = 'none';
        setTimeout(function () { return _this.recalculateStickyStyles(); });
    };
    StickyDirective.prototype.shouldShowSticky = function () {
        return this.fixedClientRect.top < this.verticalOffset || this.fixedClientRect.left < this.horizontalOffset;
    };
    StickyDirective.prototype.shouldHideSticky = function () {
        return this.fixedClientRect.top >= this.verticalOffset || this.fixedClientRect.left >= this.horizontalOffset;
    };
    StickyDirective.prototype.showSticky = function () {
        this.stickyStyle.display = this.fixedStyle.display;
        this.fixedStyle.visibility = 'hidden';
    };
    StickyDirective.prototype.hideSticky = function () {
        this.stickyStyle.display = 'none';
        this.fixedStyle.visibility = 'visible';
    };
    StickyDirective.prototype.onScroll = function () {
        if (this.shouldShowSticky()) {
            this.showSticky();
        }
        else if (this.shouldHideSticky()) {
            this.hideSticky();
        }
    };
    StickyDirective.prototype.recalculateStickyStyles = function () {
        this.stickyStyle.width = this.fixedClientRect.width + "px";
        this.stickyStyle.height = this.fixedClientRect.height + "px";
        this.stickyStyle.left = Math.max(this.horizontalOffset, this.fixedClientRect.left) + "px";
        this.stickyStyle.top = Math.max(this.verticalOffset, this.fixedClientRect.top) + "px";
    };
    StickyDirective.ctorParameters = function () { return [
        { type: ViewContainerRef },
        { type: TemplateRef },
        { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
        { type: Renderer2 }
    ]; };
    __decorate([
        Input('scSticky')
    ], StickyDirective.prototype, "config", void 0);
    StickyDirective = __decorate([
        Directive({
            selector: '[scSticky]'
        }),
        __param(2, Inject(DOCUMENT))
    ], StickyDirective);
    return StickyDirective;
}());

var TH_SELECTOR = 'tr th';
var StickyTheadDirective = /** @class */ (function (_super) {
    __extends(StickyTheadDirective, _super);
    function StickyTheadDirective(container, template, document, renderer) {
        var _this = _super.call(this, container, template, document, renderer) || this;
        _this.container = container;
        _this.template = template;
        _this.document = document;
        _this.renderer = renderer;
        return _this;
    }
    StickyTheadDirective.prototype.initializeStickyStyles = function () {
        this.stickyStyle.display = 'block';
        this.stickyStyle.overflow = 'hidden';
        _super.prototype.initializeStickyStyles.call(this);
    };
    StickyTheadDirective.prototype.recalculateStickyStyles = function () {
        _super.prototype.recalculateStickyStyles.call(this);
        var fixedThs = this.fixed.rootNodes[0].querySelectorAll(TH_SELECTOR);
        var stickyThs = this.sticky.rootNodes[0].querySelectorAll(TH_SELECTOR);
        for (var i = 0; i < fixedThs.length; i++) {
            stickyThs[i].style.width = fixedThs[i].getBoundingClientRect().width + "px";
            stickyThs[i].style.height = fixedThs[i].getBoundingClientRect().height + "px";
            stickyThs[i].style.display = 'inline-block';
        }
    };
    StickyTheadDirective.ctorParameters = function () { return [
        { type: ViewContainerRef },
        { type: TemplateRef },
        { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
        { type: Renderer2 }
    ]; };
    __decorate([
        Input('scStickyThead')
    ], StickyTheadDirective.prototype, "stickyOffset", void 0);
    StickyTheadDirective = __decorate([
        Directive({
            selector: '[scStickyThead]'
        }),
        __param(2, Inject(DOCUMENT))
    ], StickyTheadDirective);
    return StickyTheadDirective;
}(StickyDirective));

var NgxStickyCopyModule = /** @class */ (function () {
    function NgxStickyCopyModule() {
    }
    NgxStickyCopyModule = __decorate([
        NgModule({
            declarations: [StickyDirective, StickyTheadDirective],
            imports: [],
            exports: [StickyDirective, StickyTheadDirective]
        })
    ], NgxStickyCopyModule);
    return NgxStickyCopyModule;
}());

/*
 * Public API Surface of ngx-sticky-copy
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxStickyCopyModule, StickyDirective, StickyTheadDirective };
//# sourceMappingURL=ngx-sticky-copy.js.map
